unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Types, db, BufDataset, sqldb, FileUtil, Forms,
  Controls, Graphics, Dialogs, DBGrids, ExtCtrls, LCLIntf;

type

  { TColumnDragObject }

  TColumnDragObject = class(TDragControlObject)
  private
    FDragImages: TDragImageList;
  protected
    function GetDragImages: TDragImageList; override;
  public
    constructor Create(AControl: TControl); override;
    destructor Destroy; override;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    BufDataset1: TBufDataset;
    DataSource1: TDataSource;
    DBGrid1: TDBGrid;
    ImageList1: TImageList;
    Panel1: TPanel;
    procedure DBGrid1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DBGrid1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DBGrid1StartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Panel1DragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure Panel1DragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure Panel1Paint(Sender: TObject);
  private
    FGridDragColumn: TColumn;
    FDragObject: TDragObject;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TColumnDragObject }

function TColumnDragObject.GetDragImages: TDragImageList;
begin
  Result := FDragImages;
end;

constructor TColumnDragObject.Create(AControl: TControl);
var
  Bitmap: TBitmap;
  Grid: TDBGrid absolute AControl;
  ARect: TRect;
begin
  inherited Create(AControl);
  FDragImages := TDragImageList.Create(AControl);
  AlwaysShowDragImages := True;

  Bitmap := TBitmap.Create;
  ARect := Grid.CellRect(Grid.SelectedColumn.Index + 1, 0);
  Bitmap.SetSize(ARect.Right - ARect.Left, ARect.Bottom - ARect.Top);
  Bitmap.Canvas.Clear;
  Bitmap.Canvas.Clear;
  Bitmap.Canvas.TextOut(0, 0, Grid.SelectedColumn.Title.Caption);

  FDragImages.Width := Bitmap.Width;
  FDragImages.Height := Bitmap.Height;
  FDragImages.AddMasked(Bitmap, $000000);
  FDragImages.DragHotspot := Point(Bitmap.Width div 2, Bitmap.Height div 2);

  Bitmap.Free;
end;

destructor TColumnDragObject.Destroy;
begin
  FDragImages.Free;
  inherited Destroy;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  i, i2: Integer;
begin
  FGridDragColumn := nil;
  BufDataset1.CreateDataset;
  for i := 0 to 9 do
  begin
    BufDataset1.Insert;
    for i2 := 0 to BufDataset1.FieldCount - 1 do
      BufDataset1.Fields[i2].AsString := 'Item' + IntToStr(i2) + IntToStr(i);
  end;
  BufDataset1.Active := True;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FDragObject.Free;
end;

procedure TForm1.DBGrid1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  AColumn: TColumn;
  ARecordOffset: Integer;
  P: TPoint;
begin
  P := DBGrid1.MouseToCell(Point(X, Y));
  if P.y = 0 then
  begin
    DBGrid1.MouseToRecordOffset(X, Y, AColumn, ARecordOffset);
    if Assigned(AColumn) then
    begin
      FGridDragColumn := AColumn;
      DBGrid1.BeginDrag(False, 10);
    end;
  end;
end;

procedure TForm1.DBGrid1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FGridDragColumn := nil;
end;

procedure TForm1.DBGrid1StartDrag(Sender: TObject; var DragObject: TDragObject);
begin
  if not (Sender is TDBGrid) then Exit;
  if not Assigned(FGridDragColumn) then Exit;
  TDBGrid(Sender).SelectedIndex := FGridDragColumn.Index;
  if Assigned(FDragObject) then FDragObject.Free;
  DragObject := TColumnDragObject.Create(Sender as TControl);
  FDragObject := DragObject;
end;

procedure TForm1.Panel1DragDrop(Sender, Source: TObject; X, Y: Integer);
begin
  if (Source is TColumnDragObject) and Assigned(FGridDragColumn) then
  begin
    FGridDragColumn.SizePriority := 0;
    FGridDragColumn.Width := 0;
  end;
end;

procedure TForm1.Panel1DragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := Assigned(FGridDragColumn);
end;

procedure TForm1.Panel1Paint(Sender: TObject);
begin
  ImageList1.Draw(Panel1.Canvas,
                 (Panel1.ClientWidth - ImageList1.Width) div 2,
                 (Panel1.ClientHeight - ImageList1.Height) div 2,
                 0);
end;

end.

